@@ -18,25 +18,31 @@ import click
 import colorama
 
 from spsdk import SPSDK_DATA_FOLDER, SPSDKError, SPSDKValueError
-from spsdk import __version__ as spsdk_version
+from spsdk.apps.blhost_helper import progress_bar
 from spsdk.apps.elftosb_utils.sb_31_helper import RootOfTrustInfo
-from spsdk.apps.utils import (
+from spsdk.apps.utils.common_cli_options import CommandsTreeGroup, spsdk_apps_common_options
+from spsdk.apps.utils.utils import (
+    INT,
+    SPSDKAppError,
     catch_spsdk_error,
     check_destination_dir,
     check_file_exists,
-    load_configuration,
+    format_raw_data,
+    parse_file_and_size,
+    parse_hex_data,
 )
 from spsdk.dat import DebugAuthenticateResponse, DebugAuthenticationChallenge, dm_commands
 from spsdk.dat.debug_credential import DebugCredential
 from spsdk.dat.debug_mailbox import DebugMailbox
-from spsdk.debuggers.utils import DebugProbeUtils, test_ahb_access
+from spsdk.debuggers.debug_probe import DebugProbe
+from spsdk.debuggers.utils import PROBES, DebugProbeUtils, test_ahb_access
+from spsdk.utils.images import BinaryImage
+from spsdk.utils.misc import find_file, load_configuration, write_file
 
 logger = logging.getLogger(__name__)
 colorama.init()
 NXPDEBUGMBOX_DATA_FOLDER: str = os.path.join(SPSDK_DATA_FOLDER, "nxpdebugmbox")
 
-# pylint: disable=protected-access
-LOG_LEVEL_NAMES = [name.lower() for name in logging._nameToLevel]
 PROTOCOL_VERSIONS = ["1.0", "1.1", "2.0", "2.1", "2.2"]
 
 
@@ -59,12 +65,39 @@ def print_output(succeeded: bool, title: str) -> None:
 
     :param succeeded: Result of operation.
     :param title: Name of operation
+    :raises SPSDKAppError: Operation failed
     """
     if succeeded:
-        logger.info(f"{title} succeeded.")
+        click.echo(f"{title} succeeded.")
     else:
-        logger.error(f"{title} failed!")
-    click.get_current_context().exit(0 if succeeded else 1)
+        raise SPSDKAppError(f"{title} failed!")
+
+
+@contextlib.contextmanager
+def _open_debug_probe(pass_obj: Dict) -> Iterator[DebugProbe]:
+    """Method opens DebugProbe object based on input arguments.
+
+    :param pass_obj: Input dictionary with arguments.
+    :return: Active DebugProbe object.
+    :raises SPSDKError: Raised with any kind of problems with debug probe.
+    """
+    interface = pass_obj["interface"]
+    serial_no = pass_obj["serial_no"]
+    debug_probe_params = pass_obj["debug_probe_params"]
+
+    debug_probes = DebugProbeUtils.get_connected_probes(
+        interface=interface, hardware_id=serial_no, user_params=debug_probe_params
+    )
+    selected_probe = debug_probes.select_probe()
+    debug_probe = selected_probe.get_probe(debug_probe_params)
+    debug_probe.open()
+
+    try:
+        yield debug_probe
+    except SPSDKError as exc:
+        raise SPSDKError(f"Failed Debug Probe operation:({str(exc)}).") from exc
+    finally:
+        debug_probe.close()
 
 
 @contextlib.contextmanager
@@ -75,33 +108,34 @@ def _open_debugmbox(pass_obj: Dict) -> Iterator[DebugMailbox]:
     :return: Active DebugMailbox object.
     :raises SPSDKError: Raised with any kind of problems with debug probe.
     """
-    interface = pass_obj["interface"]
-    serial_no = pass_obj["serial_no"]
-    debug_probe_params = pass_obj["debug_probe_params"]
     timing = pass_obj["timing"]
     reset = pass_obj["reset"]
     operation_timeout = pass_obj["operation_timeout"]
 
-    debug_probes = DebugProbeUtils.get_connected_probes(
-        interface=interface, hardware_id=serial_no, user_params=debug_probe_params
-    )
-    selected_probe = debug_probes.select_probe()
-    debug_probe = selected_probe.get_probe(debug_probe_params)
-    debug_probe.open()
-
-    dm = DebugMailbox(
-        debug_probe=debug_probe, reset=reset, moredelay=timing, op_timeout=operation_timeout
-    )
-    try:
-        yield dm
-    except SPSDKError as exc:
-        raise SPSDKError(f"Failed Debug Mailbox command:({str(exc)}).") from exc
-    finally:
-        dm.close()
+    with _open_debug_probe(pass_obj) as debug_probe:
+        dm = DebugMailbox(
+            debug_probe=debug_probe, reset=reset, moredelay=timing, op_timeout=operation_timeout
+        )
+        try:
+            yield dm
+        except SPSDKError as exc:
+            raise SPSDKError(f"Failed Debug Mailbox command:({str(exc)}).") from exc
+        finally:
+            dm.close()
 
 
-@click.group(no_args_is_help=True)
-@click.option("-i", "--interface")
+@click.group(name="nxpdebugmbox", no_args_is_help=True, cls=CommandsTreeGroup)
+@click.option(
+    "-i",
+    "--interface",
+    type=click.Choice(list(PROBES.keys())),
+    help="Probe interface selection,if not specified, all available debug probe interfaces are used.",
+)
+@click.option(
+    "-s",
+    "--serial-no",
+    help="Debug probe hardware ID/serial number to select the probe in system.",
+)
 @click.option(
     "-p",
     "--protocol",
@@ -113,18 +147,23 @@ def _open_debugmbox(pass_obj: Dict) -> Iterator[DebugMailbox]:
     type=click.Choice(PROTOCOL_VERSIONS),
 )
 @click.option(
-    "-d",
-    "--debug",
-    "log_level",
-    metavar="LEVEL",
-    default="info",
-    help=f"Set the level of system logging output. "
-    f'Available options are: {", ".join(LOG_LEVEL_NAMES)}',
-    type=click.Choice(LOG_LEVEL_NAMES),
+    "-t",
+    "--timing",
+    type=float,
+    default=0.0,
+    help="Time of extra delay after reset sequence, defaults to 1.0 second",
+)
+@click.option(
+    "-n",
+    "--no-reset",
+    "reset",
+    is_flag=True,
+    default=True,
+    help=(
+        "Omit reset of debug mailbox during initialization,"
+        " default behavior is reset debug mailbox during initialization."
+    ),
 )
-@click.option("-t", "--timing", type=float, default=0.0)
-@click.option("-s", "--serial-no")
-@click.option("-n", "--no-reset", "reset", is_flag=True, default=True)
 @click.option(
     "-o",
     "--debug-probe-option",
@@ -136,25 +175,23 @@ def _open_debugmbox(pass_obj: Dict) -> Iterator[DebugMailbox]:
     type=int,
     default=4000,
     help="Special option to change the standard operation timeout used"
-    " for communication with debugmailbox. Default value is 4000ms.",
+    " for communication with debug mailbox. Default value is 4000ms.",
 )
-@click.version_option(spsdk_version, "-v", "--version")
-@click.help_option("--help")
+@spsdk_apps_common_options
 @click.pass_context
 def main(
     ctx: click.Context,
     interface: str,
     protocol: str,
-    log_level: str,
+    log_level: int,
     timing: float,
     serial_no: str,
     debug_probe_option: List[str],
     reset: bool,
     operation_timeout: int,
 ) -> int:
-    """NXP 'Debug Mailbox'/'Debug Credential file generator' Tool."""
-    logging.basicConfig(level=log_level.upper())
-    logger.setLevel(level=log_level.upper())
+    """Tool for working with Debug Mailbox."""
+    logging.basicConfig(level=log_level or logging.WARNING)
 
     probe_user_params = {}
     for par in debug_probe_option:
@@ -177,7 +214,7 @@ def main(
     return 0
 
 
-@main.command()
+@main.command(name="auth", no_args_is_help=True)
 @click.option("-b", "--beacon", type=int, help="Authentication beacon")
 @click.option("-c", "--certificate", help="Path to Debug Credentials.")
 @click.option("-k", "--key", help="Path to DCK private key.")
@@ -192,6 +229,7 @@ def auth(pass_obj: dict, beacon: int, certificate: str, key: str, no_exit: bool)
     """Perform the Debug Authentication."""
     try:
         logger.info("Starting Debug Authentication")
+
         with _open_debugmbox(pass_obj) as mail_box:
             with open(certificate, "rb") as f:
                 debug_cred_data = f.read()
@@ -233,7 +271,7 @@ def auth(pass_obj: dict, beacon: int, certificate: str, key: str, no_exit: bool)
                 )
                 logger.info(f"Debug Authentication ends {res_str}{colorama.Fore.RESET}.")
                 if not ahb_access_granted:
-                    click.get_current_context().exit(1)
+                    raise SPSDKAppError()
             else:
                 logger.info(
                     "Debug Authentication ends without exit and without test of AHB access."
@@ -241,10 +279,10 @@ def auth(pass_obj: dict, beacon: int, certificate: str, key: str, no_exit: bool)
 
     except SPSDKError as e:
         logger.error(f"Start Debug Mailbox failed!\n{e}")
-        click.get_current_context().exit(1)
+        raise SPSDKAppError() from e
 
 
-@main.command()
+@main.command(name="start")
 @click.pass_obj
 def start(pass_obj: dict) -> None:
     """Start DebugMailBox."""
@@ -257,7 +295,7 @@ def start(pass_obj: dict) -> None:
         print_output(result, "Start Debug Mailbox")
 
 
-@main.command()
+@main.command(name="exit")
 @click.pass_obj
 def exit(pass_obj: dict) -> None:  # pylint: disable=redefined-builtin
     """Exit DebugMailBox."""
@@ -270,7 +308,7 @@ def exit(pass_obj: dict) -> None:  # pylint: disable=redefined-builtin
         print_output(result, "Exit Debug Mailbox")
 
 
-@main.command()
+@main.command(name="erase")
 @click.pass_obj
 def erase(pass_obj: dict) -> None:
     """Erase Flash."""
@@ -283,7 +321,7 @@ def erase(pass_obj: dict) -> None:
         print_output(result, "Mass flash erase")
 
 
-@main.command()
+@main.command(name="famode")
 @click.pass_obj
 def famode(pass_obj: dict) -> None:
     """Set Fault Analysis Mode."""
@@ -296,7 +334,7 @@ def famode(pass_obj: dict) -> None:
         print_output(result, "Set fault analysis mode")
 
 
-@main.command()
+@main.command(name="ispmode", no_args_is_help=True)
 @click.option("-m", "--mode", type=int, required=True)
 @click.pass_obj
 def ispmode(pass_obj: dict, mode: int) -> None:
@@ -310,8 +348,10 @@ def ispmode(pass_obj: dict, mode: int) -> None:
         print_output(result, "Entering into ISP mode")
 
 
-@main.command()
-@click.option("-f", "--file", type=str, required=True)
+@main.command(name="blankauth", no_args_is_help=True)
+@click.option(
+    "-f", "--file", type=click.Path(), required=True, help="Path to token file (string hex format)."
+)
 @click.option(
     "-n",
     "--no-exit",
@@ -331,7 +371,7 @@ def blankauth(pass_obj: dict, file: str, no_exit: bool) -> None:
                     if not chunk:
                         break
                     token.append(int(chunk, 16))
-                f.close()
+
             dm_commands.EnterBlankDebugAuthentication(dm=mail_box).run(token)
             if not no_exit:
                 exit_response = dm_commands.ExitDebugMailbox(dm=mail_box).run()
@@ -348,7 +388,7 @@ def blankauth(pass_obj: dict, file: str, no_exit: bool) -> None:
                 )
                 logger.info(f"Debug Authentication ends {res_str}{colorama.Fore.RESET}.")
                 if not ahb_access_granted:
-                    click.get_current_context().exit(1)
+                    raise SPSDKAppError()
             else:
                 logger.info(
                     "Debug Authentication ends without exit and without test of AHB access."
@@ -356,40 +396,180 @@ def blankauth(pass_obj: dict, file: str, no_exit: bool) -> None:
 
     except SPSDKError as e:
         logger.error(colorama.Fore.RED + f"Debug authentication for Blank device failed!\n{e}")
-        click.get_current_context().exit(1)
+        raise SPSDKAppError() from e
 
 
-@main.command()
+@main.command(name="get-crp")
+@click.pass_obj
+def get_crp(pass_obj: dict) -> None:
+    """Get CRP level.
+
+    Note: This command should be called after 'start' command and with no-reset '-n' option.
+    """
+    result = False
+    try:
+        with _open_debugmbox(pass_obj) as mail_box:
+            crp_level = dm_commands.GetCRPLevel(dm=mail_box).run()[0]
+            click.echo(f"CRP level is: {crp_level}.")
+        result = True
+    finally:
+        print_output(result, "Get CRP Level")
+
+
+@main.command(name="start-debug-session")
+@click.pass_obj
+def start_debug_session(pass_obj: dict) -> None:
+    """Start debug session."""
+    result = False
+    try:
+        with _open_debugmbox(pass_obj) as mail_box:
+            dm_commands.StartDebugSession(dm=mail_box).run()
+        result = True
+    finally:
+        print_output(result, "Start debug session")
+
+
+@main.command(name="test-connection")
 @click.pass_obj
 def test_connection(pass_obj: dict) -> None:
     """Method just try if the device debug port is opened or not."""
     ahb_access_granted = False
-    try:
-
-        interface = pass_obj["interface"]
-        serial_no = pass_obj["serial_no"]
-        debug_probe_params = pass_obj["debug_probe_params"]
-        debug_probe = None
-
-        debug_probes = DebugProbeUtils.get_connected_probes(
-            interface=interface, hardware_id=serial_no, user_params=debug_probe_params
-        )
-        selected_probe = debug_probes.select_probe()
-        debug_probe = selected_probe.get_probe(debug_probe_params)
-        debug_probe.open()
-
-        debug_probe.enable_memory_interface()
-        ahb_access_granted = test_ahb_access(debug_probe)
-    except SPSDKError as exc:
-        click.echo(str(exc))
-    finally:
-        if debug_probe:
-            debug_probe.close()
+    with _open_debug_probe(pass_obj) as debug_probe:
+        try:
+            debug_probe.enable_memory_interface()
+            ahb_access_granted = test_ahb_access(debug_probe)
+        except SPSDKError as exc:
+            click.echo(str(exc))
+        finally:
             access_str = colorama.Fore.GREEN if ahb_access_granted else colorama.Fore.RED + "not-"
             click.echo(f"The device is {access_str}accessible for debugging.{colorama.Fore.RESET}")
 
 
-@main.command()
+@main.command(name="read-memory", no_args_is_help=True)
+@click.argument("address", type=INT(), required=True)
+@click.argument("byte_count", type=INT(), required=True)
+@click.argument("out_file", metavar="FILE", type=click.Path(), required=False)
+@click.option("-h", "--use-hexdump", is_flag=True, default=False, help="Use hexdump format")
+@click.pass_obj
+def read_memory(
+    pass_obj: dict,
+    address: int,
+    byte_count: int,
+    out_file: str,
+    use_hexdump: bool,
+) -> None:
+    """Reads the memory and writes it to the file or stdout.
+
+    Returns the contents of memory at the given <ADDRESS>, for a specified <BYTE_COUNT>.
+    Data are read by 4 bytes at once and are store in little endian format!
+    \b
+    ADDRESS     - starting address
+    BYTE_COUNT  - number of bytes to read
+    FILE        - store result into this file, if not specified use stdout
+    """
+    bin_image = BinaryImage("memRead", byte_count, offset=address)
+    start_addr = bin_image.aligned_start(4)
+    length = bin_image.aligned_length(4)
+
+    data = bytes()
+    with _open_debug_probe(pass_obj) as debug_probe:
+        try:
+            debug_probe.enable_memory_interface()
+            with progress_bar(
+                suppress=logger.getEffectiveLevel() > logging.INFO
+            ) as progress_callback:
+                for addr in range(start_addr, start_addr + length, 4):
+                    progress_callback(addr, start_addr + length)
+                    data += debug_probe.mem_reg_read(addr).to_bytes(4, "little")
+        except SPSDKError as exc:
+            logger.error(str(exc))
+
+    if not data:
+        click.echo("The read operation failed.")
+        return
+    if len(data) != length:
+        click.echo(
+            f"The memory wasn't read complete. It was read just first {len(data) - (address-start_addr)} Bytes."
+        )
+    # Shrink start padding data
+    data = data[address - start_addr :]
+    # Shrink end padding data
+    data = data[:byte_count]
+    if out_file:
+        write_file(data, out_file, mode="wb")
+        click.echo(f"The memory has been read and write into {out_file}")
+    else:
+        click.echo(format_raw_data(data, use_hexdump=use_hexdump))
+
+
+@main.command(name="write-memory", no_args_is_help=True)
+@click.argument("address", type=INT(), required=True)
+@click.argument("data_source", metavar="FILE[,BYTE_COUNT] | {{HEX-DATA}}", type=str, required=True)
+@click.pass_obj
+def write_memory(pass_obj: dict, address: int, data_source: str) -> None:
+    """Writes memory from a file or a hex-data.
+
+    Writes memory at <ADDRESS> from <FILE> or <HEX-DATA>
+    Writes a provided buffer to a specified <BYTE_COUNT> in memory.
+
+    \b
+    ADDRESS     - starting address
+    FILE        - write the content of this file
+    BYTE_COUNT  - if specified, load only first BYTE_COUNT number of bytes from file
+    HEX-DATA    - string of hex values: {{112233}}, {{11 22 33}}
+    """
+    try:
+        data = parse_hex_data(data_source)
+    except SPSDKError:
+        file_path, size = parse_file_and_size(data_source)
+        with open(file_path, "rb") as f:
+            data = f.read(size)
+
+    byte_count = len(data)
+    bin_image = BinaryImage("memRead", byte_count, offset=address)
+    start_addr = bin_image.aligned_start(4)
+    length = bin_image.aligned_length(4)
+
+    with _open_debug_probe(pass_obj) as debug_probe:
+        try:
+            debug_probe.enable_memory_interface()
+            start_padding = address - start_addr
+            align_data = data
+            if start_padding:
+                align_start_word = debug_probe.mem_reg_read(start_addr).to_bytes(4, "little")
+                align_data = align_start_word[:start_padding] + data
+
+            end_padding = length - byte_count - start_padding
+            if end_padding:
+                align_end_word = debug_probe.mem_reg_read(start_addr + length - 4).to_bytes(
+                    4, "little"
+                )
+                align_data = align_data + align_end_word[4 - end_padding :]
+
+            with progress_bar(
+                suppress=logger.getEffectiveLevel() > logging.INFO
+            ) as progress_callback:
+                for i, addr in enumerate(range(start_addr, start_addr + length, 4)):
+                    progress_callback(addr, start_addr + length)
+                    to_write = int.from_bytes(align_data[i * 4 : i * 4 + 4], "little")
+                    debug_probe.mem_reg_write(addr, to_write)
+                    # verify write
+                    try:
+                        verify_data = debug_probe.mem_reg_read(addr)
+                    except SPSDKError as ver_exc:
+                        raise SPSDKError("The write verification failed.") from ver_exc
+                    if to_write != verify_data:
+                        raise SPSDKError(
+                            f"Data verification failed! {hex(to_write)} != {hex(verify_data)}"
+                        )
+        except SPSDKError as exc:
+            click.echo(f"The write operation failed. Reason: {str(exc)}")
+            return
+
+    click.echo("The memory has been write successfully.")
+
+
+@main.command(name="gendc", no_args_is_help=True)
 @click.option(
     "-p",
     "--protocol",
@@ -410,14 +590,14 @@ def test_connection(pass_obj: dict) -> None:
 @click.option(
     "-c",
     "--config",
-    type=click.File("r"),
+    type=click.Path(exists=True, dir_okay=False),
     required=True,
     help="Specify YAML credential config file.",
 )
 @click.option(
     "-e",
     "--elf2sb-config",
-    type=click.File("r"),
+    type=click.Path(exists=True, dir_okay=False),
     required=False,
     help="Specify Root Of Trust from configuration file used by elf2sb tool",
 )
@@ -429,17 +609,17 @@ def test_connection(pass_obj: dict) -> None:
 )
 @click.option(
     "--plugin",
-    type=click.Path(exists=True, file_okay=True),
+    type=click.Path(exists=True, dir_okay=False),
     required=False,
     help="External python file containing a custom SignatureProvider implementation.",
 )
 @click.argument("dc_file_path", metavar="PATH", type=click.Path(file_okay=True))
 def gendc(
     protocol: str,
-    plugin: click.Path,
+    plugin: str,
     dc_file_path: str,
-    config: click.File,
-    elf2sb_config: click.File,
+    config: str,
+    elf2sb_config: str,
     force: bool,
 ) -> None:
     """Generate debug certificate (DC).
@@ -465,12 +645,18 @@ def gendc(
     check_file_exists(dc_file_path, force)
 
     logger.info("Loading configuration from yml file...")
-    yaml_content = load_configuration(config.name)
+    yaml_content = load_configuration(config)
     if elf2sb_config:
+        elf2sb_config_dir = os.path.dirname(elf2sb_config)
         logger.info("Loading configuration from elf2sb config file...")
-        rot_info = RootOfTrustInfo(load_configuration(elf2sb_config.name))  # type: ignore
-        yaml_content["rot_meta"] = rot_info.public_keys
-        yaml_content["rotk"] = rot_info.private_key
+        rot_info = RootOfTrustInfo(
+            load_configuration(elf2sb_config), search_paths=[elf2sb_config_dir]
+        )
+        yaml_content["rot_meta"] = [
+            find_file(x, search_paths=[elf2sb_config_dir]) for x in rot_info.public_keys
+        ]
+        assert rot_info.private_key
+        yaml_content["rotk"] = find_file(rot_info.private_key, search_paths=[elf2sb_config_dir])
         yaml_content["rot_id"] = rot_info.public_key_index
 
     # enforcing rot_id presence in yaml config...
@@ -486,7 +672,7 @@ def gendc(
     print_output(True, "Creating Debug credential file")
 
 
-@main.command()
+@main.command(name="get-cfg-template", no_args_is_help=True)
 @click.argument("output", metavar="PATH", type=click.Path())
 @click.option(
     "-f",
